package com.ippon.pouet.common.domain.error;

import static org.assertj.core.api.Assertions.*;

import com.ippon.pouet.common.domain.error.PouetException.PouetExceptionBuilder;
import org.junit.jupiter.api.Test;

class PouetExceptionUnitTest {

  @Test
  public void shouldGetUnmodifiableArguments() {
    assertThatThrownBy(() -> fullBuilder().build().getArguments().clear()).isExactlyInstanceOf(UnsupportedOperationException.class);
  }

  @Test
  public void shouldBuildWithoutBuilder() {
    PouetException exception = new PouetException(null);

    assertThat(exception.getCause()).isNull();
    assertThat(exception.getMessage()).isNull();
    assertThat(exception.getStatus()).isNull();
    assertThat(exception.getArguments()).isNull();
    assertThat(exception.getPouetMessage()).isNull();
  }

  @Test
  public void shouldGetExceptionInformation() {
    PouetException exception = fullBuilder().build();

    assertThat(exception.getArguments()).hasSize(2).contains(entry("key", "value"), entry("other", "test"));
    assertThat(exception.getStatus()).isEqualTo(ErrorStatus.BAD_REQUEST);
    assertThat(exception.getMessage()).isEqualTo("Error message");
    assertThat(exception.getPouetMessage()).isEqualTo(StandardMessage.INTERNAL_SERVER_ERROR);
    assertThat(exception.getCause()).isExactlyInstanceOf(RuntimeException.class);
  }

  @Test
  void shouldMapNullArgumentAsNullString() {
    assertThat(fullBuilder().argument("nullable", null).build().getArguments().get("nullable")).isEqualTo("null");
  }

  @Test
  void shouldGetObjectsToString() {
    assertThat(fullBuilder().argument("object", 4).build().getArguments().get("object")).isEqualTo("4");
  }

  private PouetExceptionBuilder fullBuilder() {
    return PouetException
      .builder(StandardMessage.INTERNAL_SERVER_ERROR)
      .argument("key", "value")
      .argument("other", "test")
      .status(ErrorStatus.BAD_REQUEST)
      .message("Error message")
      .cause(new RuntimeException());
  }
}
